@@ -1,6 +1,5 @@
 /* This file is part of GNU cflow
-   Copyright (C) 1997, 2005, 2007, 2009-2010, 2014-2016 Sergey
-   Poznyakoff
+   Copyright (C) 1997-2019 Sergey Poznyakoff
  
    GNU cflow is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -86,8 +85,8 @@ typedef struct {
 
 enum symbol_flag {
      symbol_none,
-     symbol_temp,                  /* Temporary symbol. Must be deleted after
-				      processing of the current module */
+     symbol_local,                 /* Unit-local symbol. Must be deleted after
+				      processing current compilation unit */
      symbol_parm,                  /* Parameter */
      symbol_alias                  /* Alias to another symbol */
 };
@@ -162,6 +161,7 @@ extern int brief_listing;
 extern int reverse_tree;
 extern int out_line;
 extern char *start_name;
+extern int all_functions;
 extern int max_depth;
 extern int emacs_option;
 extern int debug;
@@ -198,6 +198,7 @@ void linked_list_iterate(struct linked_list **plist,
 			 int (*itr) (void *, void *), void *data);
 void linked_list_unlink(struct linked_list *list,
 			struct linked_list_entry *ent);
+size_t linked_list_size(struct linked_list *list);
 
 int data_in_list(void *data, struct linked_list *list);
 
